#
# Base board Sample Program
#
# Copyright(C) 2024. fireflake,inc.
#

from machine import SPI, Pin
import utime

class Lps25hb_spi:
    
    # LPS25HBレジスタアドレス
    WHO_AM_I = const(0x0F)
    CTRL_REG1 = const(0x20)
    PRESS_OUT_XL = const(0x28)
    # PRESS_OUT_L = const(0x29)
    # PRESS_OUT_H = const(0x2A)
    # TEMP_OUT_L = const(0x2B)
    # TEMP_OUT_H = const(0x2C)
    
    def __init__(self, spi, cs):
        self.spi = spi
        self.cs = cs
        self._init_lps25hb()

    def _init_lps25hb(self):
        # WHO_AM_Iと名付けられたレジスタを読んで0xBDが返るのが正しいLPS25HBの動作
        self._spi_enable()
        self.spi.write(bytearray([WHO_AM_I|0x80]))
        who_am_i = self.spi.read(1)
        self._spi_disable()
        if who_am_i != b'\xBD':
            raise ValueError("LPS25HB not found")
        
        # パワーダウンをOFFにしてノーマルモードにする処理、と下記に書いてある
        # https://akizukidenshi.com/download/ds/akizuki/ae-lps25hb.pdf
        # しかしこれを実行しなくても気圧、温度の読み込みは行える模様。。。念のため書いておく
        self._spi_enable()
        self.spi.write(bytes([CTRL_REG1, 0x90]))
        self._spi_disable()

    def _read_sensor_data(self):
        # PRESS_OUT_XLアドレス（0x28）を起点に、アドレスをインクリメントしながら連続で読む
        self._spi_enable()
        self.spi.write(bytearray([PRESS_OUT_XL|0x80|0x40]))
        read_result = self.spi.read(5)  # 気圧と温度計算のため5バイト読み込み
        self._spi_disable()
        
        pressure = (read_result[2] * 65536 + read_result[1] * 256 + read_result[0]) / 4096.0
        temperature = (read_result[4] * 256 + read_result[3])
        
        if temperature >= 32768:
            temperature -= 65536
            temperature =  42.5 + temperature / 480.0

        return pressure, temperature
    
    def _spi_enable(self, b=True):
        self.cs.value(0) if b else self.cs.value(1)

    def _spi_disable(self):
        self._spi_enable(False)

def main():
    
    # SPIチャンネルの設定
    SPI0_CHANNEL = const(0)
    SPI0_BAUDRATE = const(1_000_000)
    SPI0_SCK_PIN = const(18)
    SPI0_MOSI_PIN = const(19)
    SPI0_MISO_PIN = const(16)
    SPI0_CS_PIN = const(17)

    # ベースボード側のSPIチャンネルを設定
    spi0 = SPI(SPI0_CHANNEL, baudrate = SPI0_BAUDRATE, sck = SPI0_SCK_PIN, mosi = SPI0_MOSI_PIN, miso = SPI0_MISO_PIN)
    cs0 = Pin(SPI0_CS_PIN, mode=Pin.OUT)
    
    lps25hb = Lps25hb_spi(spi0, cs0)
    
    while True:
        pressure, temperature = lps25hb._read_sensor_data()
        print('{:.0f}hPa'.format(pressure))
        print('{:.1f}C'.format(temperature))
        utime.sleep(3)

if __name__ == "__main__":
    main()
