#
# Multi Air Sensor board Sample Program
#
# Copyright(C) 2023. fireflake,inc.
#

from machine import I2C, Pin
import utime
import conf

class Dht20(object):
        
    class ReadException(Exception):
        def __init__(self):
            super().__init__('DHT20 read err.')
    
    class NotFoundException(Exception):
        def __init__(self):
            super().__init__('DHT20 not found.')
    
    def __init__(self, i2c, addr = 0x38, max_retry = 6):
        self.i2c = i2c
        self.addr = addr
        self.max_retry = max_retry
        # DHT20が見つからなかったり正しくread出来ない場合はエラー
        if self.addr in i2c.scan():
            # 試しにreadしてみる
            self.ready()
            self.read_raw_measure()
        else:
            raise self.NotFoundException()
    
    # DHT20の動作準備
    def ready(self):
        # 電源投入後に100ms待ち
        utime.sleep_ms(100)
        for i in range(self.max_retry):
            self.i2c.writeto(self.addr, bytearray(b'\x71'))
            # 0x71を送って、0x18が返ってきたら準備完了
            if self.i2c.readfrom(self.addr, 1)[0] == 0x18:
                return 0
            else:
                # 0x18が返らない場合は3つのレジスタを初期化する
                buffer = bytearray(b'\x00\x00')
                self.i2c.writeto_mem(self.addr, 0x1b, buffer)
                self.i2c.writeto_mem(self.addr, 0x1c, buffer)
                self.i2c.writeto_mem(self.addr, 0x1e, buffer)
        raise RuntimeError('DHT20 init err.')
    
    def __measure(self):
        # trigger measurementコマンドを送る
        utime.sleep_ms(10)
        self.i2c.writeto_mem(self.addr, 0xAC, bytearray(b'\x33\x00'))
        utime.sleep_ms(80)
        cnt = 0
        while (self.__read_status() & 0x80) == 0x80:
            utime.sleep_ms(1)
            if cnt >= 100:
                cnt += 1
                break
        buffer = self.i2c.readfrom(self.addr, 7)
        return buffer
    
    def __read_status(self):
        data = self.i2c.readfrom(self.addr, 1, True)
        return data[0]
        
    def read_raw_measure(self):
        if not self.addr in self.i2c.scan():
            raise self.NotFoundException()
        for i in range(self.max_retry):
            data = self.__measure()
            dcount = len(data)
            if dcount == 7:
                return data
        raise self.ReadException()
    
    def read_temp_humid(self):
        data = self.read_raw_measure()
        
        temp = 0
        temp = (temp | data[3]) << 8
        temp = (temp | data[4]) << 8
        temp = temp | data[5]
        temp = temp & 0xfffff
        temp = (temp * 200 * 10 / 1024 / 1024 - 500)/10
        
        humid = 0
        humid = (humid | data[1]) << 8
        humid = (humid | data[2]) << 8
        humid = humid | data[3]
        humid = humid >> 4
        humid = (humid * 100 * 10 / 1024 / 1024)/10
        
        return temp, humid
    
    def read_temp(self):
        temp, humid = self.read_temp_humid()
        return temp
    
    def read_humid(self): 
        temp, humid = self.read_temp_humid()
        return humid

def main():
    # マルチエアーセンサーボード側のI2Cチャンネルを設定
    i2c = I2C(conf.I2C1_CHANNEL,
              scl = Pin(conf.I2C1_SCL_PIN),
              sda = Pin(conf.I2C1_SDA_PIN),
              freq = conf.I2C1_FREQ)
    dht20 = Dht20(i2c)
    dht20.ready()
    
    while True:
        print(dht20.read_temp())
        print(dht20.read_humid())
        utime.sleep(3)

if __name__ == "__main__":
    main()
