#
# Multi Air Sensor board Sample Program
#
# Copyright(C) 2023. fireflake,inc.
#

from machine import I2C, Pin
import utime
import conf

class Lps25hb:
    
    class ReadException(Exception):
        def __init__(self):
            super().__init__('LPS25 read err.')
    
    class NotFoundException(Exception):
        def __init__(self):
            super().__init__('LPS25 not found.')
    
    def __init__(self, i2c, addr = 0x5C, max_retry = 6):
        self.i2c = i2c
        self.addr = addr
        self.max_retry = max_retry
        # LPS25が見つからなかったり正しくread出来ない場合はエラー
        exists = False
        if self.addr in i2c.scan():
            if self.is_lps25hb():
                exists = True
        if exists:
            # 試しにreadしてみる
            self.wakeup()
            self.read_raw_measure()
        else:
            raise self.NotFoundException()
    
    def is_lps25hb(self):
        # LPS25HBであれば、WHO_AM_Iと名付けられた0x0Fレジスタを1バイト読むと0xbdが返る
        self.i2c.writeto(self.addr, bytes([0x0f]))
        r = self.i2c.readfrom(self.addr, 1)
        if r == bytes([0xbd]):
            return True
        return False
    
    def wakeup(self):
        # ウェイクアップコマンドを送る
        self.i2c.writeto(self.addr, bytearray([0x20, 0x90]))

    def read_temp_press(self):
        data = self.read_raw_measure()
        press = (data[2] * 65536 + data[1] * 256 + data[0]) / 4096.0
        temp = (data[4] * 256 + data[3])
        if temp >= 32768:
            temp -= 65536
        temp =  42.5 + temp / 480.0
        return temp, press
    
    def read_raw_measure(self):
        if not self.addr in self.i2c.scan():
            raise self.NotFoundException()  
        for i in range(self.max_retry):
            # データ取得
            self.i2c.writeto(self.addr, bytes([0x28 | 0x80]))
            data = self.i2c.readfrom(self.addr, 5)
            dcount = len(data)
            if dcount == 5:
                return data
            # エラーの場合には所定回数リトライ
        # 所定回数のリトライに失敗したらエラー
        raise self.ReadException()

    def read_temp(self):
        temp, press = self.read_temp_press()
        return temp
    
    def read_press(self):
        temp, press = self.read_temp_press()
        return press

def main():
    # マルチエアーセンサーボード側のI2Cチャンネルを設定
    i2c = I2C(conf.I2C1_CHANNEL,
              scl = Pin(conf.I2C1_SCL_PIN),
              sda = Pin(conf.I2C1_SDA_PIN),
              freq = conf.I2C1_FREQ)
    lps25hb = Lps25hb(i2c)
    lps25hb.wakeup()
    
    while True:
        print(lps25hb.read_press())
        print(lps25hb.read_temp())
        utime.sleep(3)

if __name__ == "__main__":
    main()
