#
# Multi Air Sensor board Sample Program
#
# Copyright(C) 2023. fireflake,inc.
#

from machine import UART, Pin
import utime
import conf

class Mhz19c:
    
    class ReadException(Exception):
        def __init__(self):
            super().__init__('MHZ19C read err.')
    
    def __init__(self, uart, max_retry = 6):
        self.uart = uart
        self.max_retry = max_retry
        # MH-Z19Cが正しくread出来ない場合はエラー
        # 試しにreadしてみる
        self.read_co2()
    
    # 自動キャリブレーションをOFFにする
    def turn_off_auto_calibration(self):
        self.uart.write(b"\xff\x01\x79\x00\x00\x00\x00\x00\x86")
    
    # 自動キャリブレーションをONにする
    def turn_on_auto_calibration(self):
        self.uart.write(b"\xff\x01\x79\xa0\x00\x00\x00\x00\xe6")

    def read_co2(self):
        RETURN_LEN = 9  # MH-Z19C からの戻り値のデータサイズ。
        for i in range(self.max_retry):
            # UART バッファにたまっている古いデータを読み捨て
            self.uart.read(self.uart.any())
            # MH-Z19C に　"0x86- Read CO2 concentration" コマンドを送信
            self.uart.write(b"\xff\x01\x86\x00\x00\x00\x00\x00\x79")
            # 9600bps の通信で受信データが届くまで10ms +α 待つ
            utime.sleep_ms(20)
            data = self.uart.read(RETURN_LEN)
            if data != None:
                if data[0] == 0xff and data[1] == 0x86 :
                    csum = self.__checksum(data)
                    if csum == data[8]:
                        return data[2]*256 + data[3]
                    else:
                        print('MH-Z19C Data Checksum error: data:'+str(data[8])+'ChkSum:'+str(csum))
                        print(data)
                else:
                    print('MH-Z19C Data not none error')
            else:
                print('MH-Z19C Data None error')
           
            # エラーの場合には所定回数リトライ
            # WDTのタイムアウト8秒のなかである程度リトライしたいので1秒待ち x 6回リトライ
            utime.sleep(1)
            print('MH-Z19C connect error. Retry count:'+str(i))
        # Retry を繰り返して loop Exit したが retry に成功しなかった
        raise self.ReadException()
        
    def __checksum(self, data):
        csum = sum(data[1:7]) & 0xff
        csum = (0xff - csum + 1) & 0xff 
#       print('csum: {:#04x}'.format(csum))
        return csum

def main():
    # マルチエアーセンサーボード上のUARTチャンネルを設定
    uart = UART(conf.UART1_CHANNEL,
                tx = Pin(conf.UART1_TX_PIN),
                rx = Pin(conf.UART1_RX_PIN))
    uart.init(baudrate = conf.UART1_BAUDRATE,
              bits = conf.UART1_BITS,
              parity = conf.UART1_PARITYBIT,
              stop = conf.UART1_STOPBIT,
              timeout = conf.UART1_TIMEOUT)
    mhz19c = Mhz19c(uart)
    
    while True:
        print(mhz19c.read_co2())
        utime.sleep(3)

if __name__ == "__main__":
    main()
