#
# Multi Air Sensor board Sample Program
#
# Copyright(C) 2023. fireflake,inc.
#

from machine import Pin, I2C, UART, WDT
import utime
import conf
from oled import Oled
from lps25hb import Lps25hb
from mhz19c import Mhz19c
from dht20 import Dht20
import network
from ambient import Ambient

# global変数を宣言
led1 = None                  # マルチエアーセンサーボード上のLED（fatal err関数が呼ばれたときに点滅）
sw = None                    # マルチエアーセンサーボード上のタクトスイッチ
oled = None                  # OLEDのインスタンス
mhz19c = None                # MH-Z19Cのインスタンス
lps25hb = None               # LPS25HBのインスタンス
dht20 = None                 # DHT20のインスタンス
wdt = None                   # ウォッチドッグタイマのインスタンス
ambient = None               # Ambientのインスタンス
wlan = None                  # WLANのインスタンス
debug_mode = False           # デバッグモードフラグ
sw_release_flag = False      # タクトスイッチが押された後離された際に True になるフラグ
ref_sw_on_time = 0           # タクトスイッチが押された時刻
ref_sw_off_time = 0          # タクトスイッチが押され終わった時刻

# Wathcdog Timer対応 sleep関数
# sleep中でもfeedしてWDTを実行させない
def sleep_with_feed(sec):
    global debug_mode, wdt
    for i in range(sec):
        if debug_mode == False and wdt != None:
            wdt.feed()
        utime.sleep(1)

# 復旧不可能な致命的なエラー発生時のエラーハンドラ
def fatal_err(err_msg):
    global led1, sw
    global debug_mode

    # Error を Log に出力
    print('Fatal err occured:' +str(err_msg))

    # OLEDに出せたら OLED にもエラーを表示(ベストエフォート）
    oled.show_data('Fatal err occur', str(err_msg))
    utime.sleep(5)

    # エラーの時の対応
    # Debug Mode OFF
    #    再起動で復旧を試みます
    # Debug Mode ON
    #    無限ループでLED 点滅。タクトスイッチが押された場合にはハードウェアリセット
    if debug_mode:
        while True:
            led1.on()
            utime.sleep_ms(500)
            led1.off()
            utime.sleep_ms(500)
            if sw.value() is 0:
                machine.reset()
    else:
        machine.reset()  # 再起動する 

# OLED表示とprintを同時に行う
def show_text(line1='', line2='', line3='', line4=''):
    global oled
    print(line1, line2, line3, line4)
    oled.show_data(line1, line2, line3, line4)

# 稼働中にタクトスイッチが押された・離された時の処理
def sw_callback(arg):
    # 長押し対応処理
    global sw_release_flag, ref_sw_on_time, ref_sw_off_time

    # SWが押されたら時の処理
    if sw.value() is 0:
        print('Sw pressureed')
        ref_sw_on_time = utime.ticks_ms()   # SW が押された時刻を取得

    # SWが押されたら時の処理
    if sw.value() is 1:
        print('Sw Released')
        ref_sw_off_time = utime.ticks_ms()  # SW が離された時刻を取得
        sw_release_flag = True              # スイッチが離されたフラグを立てる

# センサーから環境データを取得
def get_sensor_data():
    global dht20, lps25hb, mhz19c
    try:
        # センサーからデータ取得
        temp, humid = dht20.read_temp_humid()
        pressure = lps25hb.read_press()
        co2 = mhz19c.read_co2()
    except Exception as e:
        fatal_err(e)
    return temp, humid, pressure, co2

# 環境データを文字列に加工して出力
def show_sensor_data(temp, humid, pressure, co2):
    try:
        # 取得したデータをOLEDに表示させる
        str_temp = '{:.1f}C'.format(temp)
        str_humid = '{:.1f}%'.format(humid)
        str_pressure = '{:.0f}hPa'.format(pressure)
        str_co2 = '{:.0f}ppm'.format(co2)
        show_text('Temp:  ' + str_temp, 'Humid: ' + str_humid, 'Press: ' + str_pressure, 'CO2:   ' + str_co2)
    except Exception as e:
        fatal_err(e)  

# 環境データをAmbientにアップロード
def send_sensor_data_to_ambient(temp, humid, pressure, co2):
    global ambient
    response = None
    try:
        response = ambient.send({'d1': temp, 'd2': humid, 'd3': pressure, 'd4': co2});
    except Exception as e:
        fatal_err('upload failed')
        
    if response == None:
        fatal_err('upload failed')
    status_code = str(response.status_code)
    response.close()
    
    print('Ambient response code: ' + status_code)
    
    # レスポンスコードが200以外ならエラー扱い
    if status_code != '200':
        fatal_err('Ambient ret ' + status_code)

# 一瞬だけLEDを点滅させる
def blink_led(led):
    led.on()
    utime.sleep_ms(50)
    led.off()

# Wi-Fi接続
def connect_to_wlan():
    global wlan
    wlan.connect(conf.WIFI_SSID, conf.WIFI_PASSWORD)
    wlan_connect_max_wait = conf.WIFI_CONNECT_MAX_WAIT
    while wlan_connect_max_wait > 0:
        # wlanのステータスコードは下記資料を参照
        # Connecting to the Internet with Raspberry Pi Pico W - 3.6.1. Connection status codes
        # https://datasheets.raspberrypi.com/picow/connecting-to-the-internet-with-pico-w.pdf
        if wlan.status() < 0 or wlan.status() >= 3:
            break
        wlan_connect_max_wait -= 1
        show_text('WiFi connecting')
        sleep_with_feed(1)
    # Handle connection error
    if wlan.status() != 3:
        fatal_err('WiFi conn err')
    else:
        show_text('WiFi connected')

def main():    
    global led1, sw
    global oled
    global mhz19c, lps25hb, dht20
    global debug_mode
    global wdt
    global ambient
    global wlan
    global ref_sw_on_time, ref_sw_off_time
    global sw_release_flag

    #初期化フェーズは debug mode 相当で動作（致命傷があっても再起動せずにエラー表示して停止）
    debug_mode = True

    # Pico W本体およびマルチエアーセンサーボード上のLEDのGPIO初期化
    led1 = Pin(conf.GPIO_11, Pin.OUT, value=0)  # LED1
    led2 = Pin(conf.GPIO_12, Pin.OUT, value=0)  # LED2
    led3 = Pin(conf.GPIO_13, Pin.OUT, value=0)  # LED3
    
    # タクトスイッチのピンを設定設定
    sw = Pin(conf.GPIO_10, Pin.IN, Pin.PULL_UP)

    # マルチエアーセンサーボード上のUARTチャンネルを設定
    uart = UART(conf.UART1_CHANNEL,
                baudrate=conf.UART1_BAUDRATE,
                tx = Pin(conf.UART1_TX_PIN),
                rx = Pin(conf.UART1_RX_PIN))
    
    # マルチエアーセンサーボード側のI2Cチャンネルを設定
    i2c = I2C(conf.I2C1_CHANNEL,
              scl = Pin(conf.I2C1_SCL_PIN),
              sda = Pin(conf.I2C1_SDA_PIN),
              freq = conf.I2C1_FREQ)

    # I2C 接続の OLEDのインスタンス生成
    # インスタンス生成時の動作チェックに失敗したらfatal errorとして処理
    try:
        oled = Oled(i2c)
        show_text('Welcome...')
        utime.sleep(1)
    except Exception as e:
        fatal_err(e)

    # センサーのインスタンス生成
    # インスタンス生成時の動作チェックに失敗したらfatal errorとして処理
    try:
        dht20 = Dht20(i2c)
        lps25hb = Lps25hb(i2c)
        # CO2 Sensor
        mhz19c = Mhz19c(uart)
        mhz19c.turn_off_auto_calibration()   # 自動キャリブレーションをOFF
    except Exception as e:
        fatal_err(e)

    # 接続したいSSIDのWi-Fiが周囲にあるか確認
    # 見つからないならfatal errorとして処理
    wlan = network.WLAN(network.STA_IF)
    wlan.active(True)
    result = wlan.scan()
    ssid_flag = False
    for data in result:
        if data[0].decode() == conf.WIFI_SSID:
            ssid_flag = True
            break
    if not ssid_flag:
        fatal_err('WiFi not found')

    # 初期化フェーズが終わったので Debug Modeは一旦ここで終了
    debug_mode = False

    # 起動時にタクトスイッチが押されているかチェック
    # 押されていたらデバッグモードとして動作する。
    # Debug Mode では watchdog タイマーが無効化されるため開発中に breakpoint で止まった際に
    # リセットがかからないといった問題を回避できます。
    if sw.value() is 0:
        debug_mode = True
        show_text('debug mode', 'Enabled', 'WDT Disabled')
        utime.sleep(5)
        while sw.value() == 0:
            pass

    # タクトスイッチが押された際に呼び出される割り込みハンドラの設定
    sw_release_flag = False
    sw.irq(sw_callback, trigger=Pin.IRQ_RISING|Pin.IRQ_FALLING)
    
    # WiFi接続
    connect_to_wlan()
    # Ambientの接続情報を設定
    ambient = Ambient(conf.AMBIENT_CHANNEL_ID, conf.AMBIENT_WRITE_KEY)
    
    # 環境データ
    temp = -999      # 温度データ
    humid = -999     # 湿度データ
    pressure = -999  # 気圧データ
    co2 = -999       # CO2データ
    
    # デバッグモードでなければWDTをセット
    wdt = None if debug_mode else WDT(timeout=8000)

    counter = 0
    while True:
        counter = counter +1

        # MEASUREMENT_LOOP_WAIT で設定された時間間隔でセンサーデータをサンプリング
        if counter  % conf.MEASUREMENT_LOOP_WAIT == 0:
            (temp, humid, pressure, co2) = get_sensor_data()
            show_sensor_data(temp, humid, pressure, co2)
            blink_led(led3)

        # AMBIENT_LOOP_WAIT で設定された時間間隔でセンサーデータをAmbientにアップロード
        if counter  % conf.AMBIENT_LOOP_WAIT == 0:
            # Wi-Fiステータスがconnection successfulになっていなければ再接続を試みる
            if wlan.status() != 3:
                wlan.disconnect()
                connect_to_wlan()
            # Send data to ambient
            send_sensor_data_to_ambient(temp, humid, pressure, co2)
            blink_led(led2)

        # キーをチェック。押したキーが離されたときに sw_release_flag が True になるので
        # 押された時間を計算、５秒間以上押されて長押しになっている場合に長押しの処理を実施
        if sw_release_flag :
            print('checking sw intervals')
            # タクトスイッチが5秒以上長押しされたらMH-Z19Cの自動キャリブレーションをONにする
            if utime.ticks_diff(ref_sw_off_time, ref_sw_on_time) >= 5000:
                print('long pressure enabled')
                mhz19c.turn_on_auto_calibration()
                show_text('CO2 auto', 'calibration on')
                sleep_with_feed(1)
            # SW が離された処理が終わったのでフラグをクリア
            sw_release_flag = False

        # 次の周期まで sleep
        sleep_with_feed(1)

if __name__ == "__main__":
    main()
