#
# Plant Env Sensor board Sample Program
#
# Copyright(C) 2024. fireflake,inc.
#

from machine import I2C, Pin
import conf
import utime

class BH1750:
    PWR_OFF = 0x00
    PWR_ON = 0x01
    RESET = 0x07
    CONT_LOWRES = 0x13
    CONT_HIRES_1 = 0x10
    CONT_HIRES_2 = 0x11
    ONCE_HIRES_1 = 0x20
    ONCE_HIRES_2 = 0x21
    ONCE_LOWRES = 0x23

    def __init__(self, bus, addr=0x23):
        self.bus = bus
        self.addr = addr
        self.set_mode(self.PWR_OFF)
        self.set_mode(self.RESET)

    def set_mode(self, mode):
        self.bus.writeto(self.addr, bytes([mode]))

    def measure_illuminance(self, mode):
        if mode != getattr(self, 'mode', None):
            self.set_mode(mode)
        utime.sleep_ms(24 if mode in (self.CONT_LOWRES, self.ONCE_LOWRES) else 180)
        data = self.bus.readfrom(self.addr, 2)
        factor = 2.0 if mode in (self.CONT_HIRES_2, self.ONCE_HIRES_2) else 1.0
        return (data[0] << 8 | data[1]) / (1.2 * factor)

def main():
    # 植物生育環境計測ボード側のI2C設定
    i2c_1 = I2C(conf.I2C1_CHANNEL, scl = Pin(conf.I2C1_SCL_PIN), sda = Pin(conf.I2C1_SDA_PIN), freq = conf.I2C1_FREQ)
    
    ## ここではSEN0562を使って照度測定と日射量計算を行う方法を記載
    # 製品wiki https://wiki.dfrobot.com/SKU_SEN0562_Gravity_I2C_Waterproof_Ambient_Light_Sensor_1_65535lx
    # センサー本体のデータシート https://components101.com/sites/default/files/component_datasheet/BH1750.pdf
    bh1750 = BH1750(i2c_1, conf.BH1750_ADDRESS)
    illuminance = bh1750.measure_illuminance(BH1750.ONCE_HIRES_1)  # 照度測定だけならここまででOK
    
    # 照度（lux）を日射量（kW/m-2）に簡易的に変換する場合は以下のようにする
    # 1.46は550-555nmでの補正値である事に注意、参考 http://www.photosynthesis.jp/light.html
    SOLAR_RADIATION_CONVERSION = 1.46
    MILLI_KILO_CONVERSION = 1000 * 1000
    solar_radiation = illuminance * SOLAR_RADIATION_CONVERSION / MILLI_KILO_CONVERSION
    print('Solar Radiation: {:.3f}kW/m2'.format(solar_radiation))

    # 積算日射量のための変数、定数
    total_solar_radiation = 0
    MINUTE_IN_HOUR = 60
    MEGA_JOULE_CONVERSION = 3.6
    print('Total Solar Radiation: {:.3f}MJ/m2'.format(total_solar_radiation))
    
    while True:
        # 照度測定
        illuminance = bh1750.measure_illuminance(BH1750.ONCE_HIRES_1)
        solar_radiation = illuminance * SOLAR_RADIATION_CONVERSION / MILLI_KILO_CONVERSION
        
        # 1分毎にループする事を前提に積算日射量を算出、単位はMJ/m2
        total_solar_radiation += (solar_radiation / MINUTE_IN_HOUR) * MEGA_JOULE_CONVERSION
        # 暫定で10MJを越えたらゼロリセット、実際に日射比例潅水に使う場合は適切に閾値を設定する事
        if total_solar_radiation > 10:
            total_solar_radiation = 0
        
        print('Solar Radiation: {:.3f}kW/m2'.format(solar_radiation))
        print('Total Solar Radiation: {:.3f}MJ/m2'.format(total_solar_radiation))
            
        utime.sleep(60)  # 1分毎にループ

if __name__ == "__main__":
    main()
