#
# Plant Env Sensor board Sample Program
#
# Copyright(C) 2024. fireflake,inc.
#

import conf
from machine import Pin, ADC
import utime

def main():
    ## ADC1, ADC2共通パラメータ
    MAX_READ_VOLTAGE = const(3.3) # Pico WのADCで読み込める最大電圧
    RAW_SCALE = const(65535)  # Pico WのADCの分解能は12bitだが、ADC.read_u16()で読んだ生の値は0-65535にスケーリングされるため
    VOLTAGE_SCALE = MAX_READ_VOLTAGE / RAW_SCALE  # 3.3Vの範囲で0-65535のスケールを表現させる

    ## ADC（3.3V）ブロック（ADC2チャンネル）
    # 土壌水分センサーのSEN0308の接続を想定
    # SEN0308は植物生育環境計測ボード上のADCに接続するのでその設定を行う
    analog_sensor_2 = Pin(conf.GPIO_15, Pin.OUT)  # ADC（3.3V）ブロックに接続するセンサーに対する電源制御のための変数、GPIOは15で固定
    analog_sensor_2.value(conf.ANALOG_SENSOR_OFF)  # ON時の電源電圧は3.3V
    adc_2 = ADC(conf.ADC_CHANNEL2)  # analog_sensor_2がONになっていないとADC2で電圧測定出来ない事に注意

    # SEN0308からの電圧値を取得
    analog_sensor_2.value(conf.ANALOG_SENSOR_ON)  # 読み込み前にON
    utime.sleep_ms(conf.PREHEAT_WAIT)  # プレヒート待ち時間、接続するセンサーを変更する場合はそれに合わせて待ち時間も変更する事
    adc_2_voltage = adc_2.read_u16() * VOLTAGE_SCALE  # 入力値を0-65535の範囲で読み込んで、それを電圧変換
    analog_sensor_2.value(conf.ANALOG_SENSOR_OFF)  # 読み込み後にOFF
    print('ADC2: {:.3f}V'.format(adc_2_voltage))

    # 測定した電圧を土壌水分量に変換
    # 0-3Vで土壌水分を表す 参考 https://www.dfrobot.com/product-2054.html
    # 以下は、最大出力が3Vの土壌水分センサーのためのごく簡易的な変換式になる
    # 正確に変換するには測定する土壌に合わせたキャリブレーション（電圧と土壌水分の変換式作成）が必要となる
    # 変換式作成の参考資料 http://soillab.ag.saga-u.ac.jp/com/calib1.pdf
    SOIL_MOISTURE_SENSOR_MAX_VOLTAGE = 3
    PERCENTAGE_CONVERSION = 100
    PERCENTAGE_MAX_VALUE = 100
    soll_moisture = PERCENTAGE_MAX_VALUE - (adc_2_voltage / SOIL_MOISTURE_SENSOR_MAX_VOLTAGE * PERCENTAGE_CONVERSION)
    print('Soil Moisture: {:.1f}%'.format(soll_moisture))

    ## ADC（5V）ブロック（ADC1チャンネル）
    # ADC（5V）ブロックは分圧抵抗を入れて使うものだが、初期は植物生育環境計測ボードに分圧抵抗が入っていないので自分でハンダ付けして入れる事
    # ADC（5V）ブロックの詳細は植物生育環境計測ボードの仕様書やユーザーガイドを参照
    analog_sensor_1 = Pin(conf.GPIO_14, Pin.OUT)  # ADC（5V）ブロックに接続するセンサーに対する電源制御のための変数、GPIOは14で固定
    analog_sensor_1.value(conf.ANALOG_SENSOR_OFF)  # ON時の電源電圧は5Vのため、センサーからの出力電圧がPico Wが受けられる3.3Vを上回らないように注意
    adc_1 = ADC(conf.ADC_CHANNEL1)  # analog_sensor_1がONになっていないとADC1で電圧測定出来ない事に注意

    # ADC1に繋いだセンサーからの電圧値を取得
    analog_sensor_1.value(conf.ANALOG_SENSOR_ON)  # 読み込み前にON
    utime.sleep_ms(conf.PREHEAT_WAIT)  # プレヒート待ち時間、接続するセンサーを変更する場合はそれに合わせて待ち時間も変更する事
    adc_1_voltage = adc_1.read_u16() * VOLTAGE_SCALE  # 入力値を0-65535の範囲で読み込んで、それを電圧変換
    analog_sensor_1.value(conf.ANALOG_SENSOR_OFF)  # 読み込み後にOFF
    print('ADC1: {:.3f}V'.format(adc_1_voltage))

if __name__ == "__main__":
    main()
