#
# Plant Env Sensor board Sample Program
#
# Copyright(C) 2024. fireflake,inc.
#

import conf
from machine import Pin, I2C, ADC
from oled import Oled
import utime
import onewire, ds18x20
from dht20 import Dht20
from bh1750 import BH1750

def main():
    # 植物生育環境計測ボード側LEDセット
    led_green = Pin(conf.GPIO_9, Pin.OUT)
    led_yellow = Pin(conf.GPIO_10, Pin.OUT)
    led_red = Pin(conf.GPIO_11, Pin.OUT)
    led_green.off()
    led_yellow.off()
    led_red.off()

    ## I2C設定
    # 植物生育環境計測ボード側のI2C設定
    i2c_1 = I2C(conf.I2C1_CHANNEL, scl = Pin(conf.I2C1_SCL_PIN), sda = Pin(conf.I2C1_SDA_PIN), freq = conf.I2C1_FREQ)
    # ベースボード側のI2C設定
    i2c_0 = I2C(conf.I2C0_CHANNEL, scl = Pin(conf.I2C0_SCL_PIN), sda = Pin(conf.I2C0_SDA_PIN), freq = conf.I2C0_FREQ)
    # I2Cに接続するセンサーが繋がっていなければ赤LED点灯
    if not conf.BH1750_ADDRESS in i2c_1.scan():
        led_red.on()
        raise RuntimeError("BH1750 is not found.")   
    if not conf.OLED_ADDRESS in i2c_1.scan():
        led_red.on()
        raise RuntimeError("OLED is not found.")  
    if not conf.DHT20_ADDRESS in i2c_0.scan():
        led_red.on()
        raise RuntimeError("DHT20 is not found.")  

    # OLEDに開始を表示
    oled = Oled(i2c_1)
    oled.show_data('starting...')
    print('---starting---')

    ## 植物生育環境計測ボード上のタクトスイッチの動作設定
    # 左側の白いタクトスイッチ
    def sw_white_callback(arg):
        # 押された時の動作
        print('white sw pushed!')
        oled.show_data('white sw', 'pushed!')
        utime.sleep(1)
        
    sw_white = Pin(conf.GPIO_12, Pin.IN, Pin.PULL_UP)
    sw_white.irq(sw_white_callback, trigger=Pin.IRQ_FALLING)

    # 右側の黒いタクトスイッチ
    def sw_black_callback(arg):
        # 押された時の動作
        print('black sw pushed!')
        oled.show_data('black sw', 'pushed!')
        utime.sleep(1)

    sw_black = Pin(conf.GPIO_13, Pin.IN, Pin.PULL_UP)
    sw_black.irq(sw_black_callback, trigger=Pin.IRQ_FALLING)

    ## 照度センサー
    # 製品wiki https://wiki.dfrobot.com/SKU_SEN0562_Gravity_I2C_Waterproof_Ambient_Light_Sensor_1_65535lx
    # センサー本体のデータシート https://components101.com/sites/default/files/component_datasheet/BH1750.pdf
    bh1750 = BH1750(i2c_1, conf.BH1750_ADDRESS)
    illuminance = bh1750.measure_illuminance(BH1750.ONCE_HIRES_1)  # 照度測定だけならここまででOK
    # 照度（lux）を日射量（kW/m-2）に簡易的に変換する場合は以下のようにする
    # 1.46は550-555nmでの補正値である事に注意、参考 http://www.photosynthesis.jp/light.html
    SOLAR_RADIATION_CONVERSION = 1.46
    MILLI_KILO_CONVERSION = 1000 * 1000
    solar_radiation = illuminance * SOLAR_RADIATION_CONVERSION / MILLI_KILO_CONVERSION
    print('Solar Radiation: {:.3f}kW/m2'.format(solar_radiation))

    # 積算日射量のための変数、定数
    total_solar_radiation = 0
    MINUTE_IN_HOUR = 60
    MEGA_JOULE_CONVERSION = 3.6
    print('Total Solar Radiation: {:.3f}MJ/m2'.format(total_solar_radiation))

    ## 1-wireセンサー（土壌温度センサー）
    # 参考 https://micropython-docs-ja.readthedocs.io/ja/latest/esp8266/tutorial/onewire.html
    one_wire_data = Pin(conf.GPIO_8)
    ds = ds18x20.DS18X20(onewire.OneWire(one_wire_data))
    roms = ds.scan()
    # 1-wireセンサーが繋がっていなければ赤LED点灯
    if not roms:
        led_red.on()
        raise RuntimeError("1-wire sensor is not found.")
    ds.convert_temp()
    utime.sleep_ms(conf.ONE_WIRE_WAIT)
    ds18x20_1_temperature = ds.read_temp(roms[0])
    #ds18x20_2_temperature = ds.read_temp(roms[1])  # 植物生育環境計測ボードにDS18X20の2個目を付ける場合はこのコメントを外す
    print('Soil Temperature: {:.1f}C'.format(ds18x20_1_temperature))

    ## 土壌水分センサー
    # 土壌水分センサーは植物生育環境計測ボード上のADCに接続するので、まずADCの設定を行う
    MAX_READ_VOLTAGE = const(3.3) # Pico WのADCで読み込める最大電圧
    RAW_SCALE = const(65535)  # Pico WのADCの分解能は12bitだが、ADC.read_u16()で読んだ生の値は0-65535にスケーリングされるため
    VOLTAGE_SCALE = MAX_READ_VOLTAGE / RAW_SCALE  # 3.3Vの範囲で0-65535のスケールを表現させる

    # 土壌水分センサーはADC2に接続するので、ここではADC2の設定のみ記載
    analog_sensor_2 = Pin(conf.GPIO_15, Pin.OUT)  # ADC（3.3V）ブロックに接続するセンサーに対する電源制御のための変数、GPIOは15で固定
    analog_sensor_2.value(conf.ANALOG_SENSOR_OFF)  # ON時の電源電圧は3.3V
    adc_2 = ADC(conf.ADC_CHANNEL2)  # analog_sensor_2がONになっていないとADC2で電圧測定出来ない事に注意

    # 土壌水分センサーからの電圧値を取得
    analog_sensor_2.value(conf.ANALOG_SENSOR_ON)  # 読み込み前にON
    utime.sleep_ms(conf.PREHEAT_WAIT)  # プレヒート待ち時間、接続するセンサーを変更する場合はそれに合わせて待ち時間も変更する事
    adc_2_voltage = adc_2.read_u16() * VOLTAGE_SCALE  # 入力値を0-65535の範囲で読み込んで、それを電圧変換
    analog_sensor_2.value(conf.ANALOG_SENSOR_OFF)  # 読み込み後にOFF

    # 測定した電圧を土壌水分量に変換
    # 0-3Vで土壌水分を表す 参考 https://www.dfrobot.com/product-2054.html
    # 以下は、最大出力が3Vの土壌水分センサーのためのごく簡易的な変換式になる
    # 正確に変換するには測定する土壌に合わせたキャリブレーション（電圧と土壌水分の変換式作成）が必要となる
    # 変換式作成の参考資料 http://soillab.ag.saga-u.ac.jp/com/calib1.pdf
    SOIL_MOISTURE_SENSOR_MAX_VOLTAGE = 3
    PERCENTAGE_CONVERSION = 100
    PERCENTAGE_MAX_VALUE = 100
    soll_moisture = PERCENTAGE_MAX_VALUE - (adc_2_voltage / SOIL_MOISTURE_SENSOR_MAX_VOLTAGE * PERCENTAGE_CONVERSION)
    print('Soil Moisture: {:.1f}%'.format(soll_moisture))

    ## 温湿度センサー
    dht20 = Dht20(i2c_0)
    dht20.ready()
    air_temperature = dht20.read_temp()
    air_humidity =dht20.read_humid()
    print('Air Temperature: {:.1f}C'.format(air_temperature))
    print('Air Humidity: {:.1f}%'.format(air_humidity))

    loop_count = 0
    # ループ動作開始
    while True:
        loop_count += 1
        # 1分毎に回すための基準
        criterion_time = utime.time()  # 現在の時間を秒単位で取得
        
        # 照度センサー
        illuminance = bh1750.measure_illuminance(BH1750.ONCE_HIRES_1)
        solar_radiation = illuminance * SOLAR_RADIATION_CONVERSION / MILLI_KILO_CONVERSION
        
        # 1分毎にループする事を前提に積算日射量を算出、単位はMJ/m2
        total_solar_radiation += (solar_radiation / MINUTE_IN_HOUR) * MEGA_JOULE_CONVERSION
        # 暫定で1MJを越えたらゼロリセット、実際に日射比例潅水に使う場合は適切に閾値を設定する事
        if total_solar_radiation > 1:
            total_solar_radiation = 0
        
        # 土壌温度センサー
        ds.convert_temp()
        utime.sleep_ms(conf.ONE_WIRE_WAIT)
        ds18x20_1_temperature = ds.read_temp(roms[0])
        
        # 土壌水分センサー
        analog_sensor_2.value(conf.ANALOG_SENSOR_ON)  # 読み込み前にON
        utime.sleep_ms(conf.PREHEAT_WAIT)  # プレヒート待ち時間、接続するセンサーを変更する場合はそれに合わせて待ち時間も変更する事
        adc_2_voltage = adc_2.read_u16() * VOLTAGE_SCALE  # 入力値を0-65535の範囲で読み込んで、それを電圧変換
        analog_sensor_2.value(conf.ANALOG_SENSOR_OFF)  # 読み込み後にOFF
        soll_moisture = PERCENTAGE_MAX_VALUE - (adc_2_voltage / SOIL_MOISTURE_SENSOR_MAX_VOLTAGE * PERCENTAGE_CONVERSION)
        
        # 温湿度センサー
        air_temperature = dht20.read_temp()
        air_humidity =dht20.read_humid()

        # コンソールに一括表示
        print('---Loop time: ' + str(loop_count) + '---')
        print('Solar Radiation: {:.3f}kW/m2'.format(solar_radiation))
        print('Total Solar Radiation: {:.3f}MJ/m2'.format(total_solar_radiation))
        print('Soil Temperature: {:.1f}C'.format(ds18x20_1_temperature))
        print('Soil Moisture: {:.1f}%'.format(soll_moisture))
        print('Air Temperature: {:.1f}C'.format(air_temperature))
        print('Air Humidity: {:.1f}%'.format(air_humidity))

        # OLEDには表示行数の都合から3回に分けて表示
        # RAD=日射量、TRAD=積算日射量（total solar radiation）
        oled.show_data('RAD   {:.3f}kW/m2'.format(solar_radiation), 'TRAD  {:.3f}MJ/m2'.format(total_solar_radiation))
        utime.sleep(conf.OLED_SHOW_WAIT)
        # AIR TEMP=気温、AIR HUMID=湿度
        oled.show_data('AIR TEMP   {:.1f}C'.format(air_temperature), 'AIR HUMID  {:.1f}%'.format(air_humidity))
        utime.sleep(conf.OLED_SHOW_WAIT)
        # SOIL TEMP=土壌温度、SOIL MOIST=土壌水分
        oled.show_data('SOIL TEMP  {:.1f}C'.format(ds18x20_1_temperature), 'SOIL MOIST {:.1f}%'.format(soll_moisture))

        # 基準時間からMAIN_LOOP_WAIT分（初期設定は60秒）だけ時間が経ったら次のループへ
        while utime.time() - criterion_time < conf.MAIN_LOOP_WAIT:
                utime.sleep(1)

if __name__ == "__main__":
    main()
